# Tying it all together, use lists, dictionaries, named tuples and sets to manipulate data
#

import collections
from pprint import pprint

# Create a named tuple
Count = collections.namedtuple('Count', 'gold silver bronze total')

# Initialise our country/medal data dictionary
data = {}
with open("medals.csv") as f:
    for line in f.readlines():
        fields = line.rstrip().split(',')           # Strip the newline then split the string by comma

        # The fields are:
        # 0: Rank
        # 1: Country name
        # 2: Number of gold medals
        # 3: Number of silver medals
        # 4: Number of bronze medals
        # 5: Number of total medals
        rank = fields[0]
        country = fields[1]                         # For convenience, save the country name
        counts_int = [int(s) for s in fields[2:]]   # Convert to integers
        counts = Count(*counts_int)                 # Now convert to a named tuple
        data[country] = counts                      # Store it in a dictionary
        
# Sort the dictionary into a list of the top countries, based on their total or gold medal count
top_totals = sorted(data, key = lambda country : data[country].total, reverse = True)
top_golds = sorted(data, key = lambda country : data[country].gold, reverse = True)

# Print the top medal countries
print "Top 10 medal countries:"
for country in top_totals[:10]:
    print "%30s = %d" % (country, data[country].total)

print
print "Top 10 gold medal countries:"
for country in top_golds[:10]:
    print "%30s = %d" % (country, data[country].gold)

# Now create sets of these two lists of countries and work out the intersection
# ie. those countries that are in both sets
set_totals = set(top_totals[:10])
set_golds = set(top_golds[:10])
print
print "Countries with top 10 gold and top 10 medal counts:"
for country in set_totals.intersection(set_golds):
    print "%30s = %d and %d" % (country, data[country].total, data[country].gold)

# Now create a list of countries that only won a single bronze medal
one_bronze = [country for country in data.keys() if data[country].bronze == 1 and data[country].total == 1]
print
print "Countries that only won a single bronze:"
for country in one_bronze:
    print "\t", country

